#include <stdio.h>
#include <stdlib.h>

// Code used to show the equivalence between the libc rand LCG
// implementation, which uses the and bit-operator & with the constant
// 2147483647, and our own custom implementation, which uses the
// modulus operator % with the constant 2147483648.
//
// Discussed in the following lecture:
// - https://teaching.leonardotamiano.xyz/cns/labs/lab_01_breaking_prng_part_1/
//
// To compile:
// gcc example_4_rand_equivalence.c -o example_4_rand_equivalence

int32_t state;

int myrand(void) 
{
  int32_t val = ((state * 1103515245U) + 12345U) % 2147483648;
  state = val;
  return val;
}

void mysrand(int32_t seed)
{
  state = seed;
}


int main() {
  // setup simple LCG
  char state1[8];
  initstate(1337, state1, 0);
  setstate(state1);

  // init default LCG
  srand(1337);

  // init mine LCG
  mysrand(1337);

  // test if default LCG is equivalent to mine LCG
  while (rand() == myrand());

  return 1;
}
